package at.ac.uibk.gitsearch.service.impl;

import at.ac.uibk.gitsearch.domain.Statistics;
import at.ac.uibk.gitsearch.repository.StatisticsRepository;
import at.ac.uibk.gitsearch.repository.search.StatisticsSearchRepository;
import at.ac.uibk.gitsearch.service.StatisticsService;
import at.ac.uibk.gitsearch.service.dto.StatisticsDTO;
import at.ac.uibk.gitsearch.service.mapper.StatisticsMapper;
import java.util.Optional;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * Service Implementation for managing {@link Statistics}.
 */
@Service
@Transactional
public class StatisticsServiceImpl implements StatisticsService {

    private final Logger log = LoggerFactory.getLogger(StatisticsServiceImpl.class);

    private final StatisticsRepository statisticsRepository;

    private final StatisticsMapper statisticsMapper;

    private final StatisticsSearchRepository statisticsSearchRepository;

    public StatisticsServiceImpl(
        StatisticsRepository statisticsRepository,
        StatisticsMapper statisticsMapper,
        StatisticsSearchRepository statisticsSearchRepository
    ) {
        this.statisticsRepository = statisticsRepository;
        this.statisticsMapper = statisticsMapper;
        this.statisticsSearchRepository = statisticsSearchRepository;
    }

    @Override
    @SuppressWarnings("PMD.AvoidCatchingGenericException")
    public StatisticsDTO save(StatisticsDTO statisticsDTO) {
        log.debug("Request to save Statistics : {}", statisticsDTO);
        Statistics statistics = statisticsMapper.toEntity(statisticsDTO);
        statistics = statisticsRepository.save(statistics);
        StatisticsDTO result = statisticsMapper.toDto(statistics);
        // TODO temporary fix: statistics has a problem with complex exercise ids (e.g. 10:if1/exercise1) :-)
        try {
            statisticsSearchRepository.save(statistics);
        } catch (Exception e) {
            log.error("Please fix this problem! ExerciseIds are not numeric any more!", e);
        }
        return result;
    }

    @Override
    @Transactional(readOnly = true)
    public Page<StatisticsDTO> findAll(Pageable pageable) {
        log.debug("Request to get all Statistics");
        return statisticsRepository.findAll(pageable).map(statisticsMapper::toDto);
    }

    @Override
    @Transactional(readOnly = true)
    public Optional<StatisticsDTO> findOne(Long id) {
        log.debug("Request to get Statistics : {}", id);
        return statisticsRepository.findById(id).map(statisticsMapper::toDto);
    }

    @Override
    public Optional<StatisticsDTO> findOneByExerciseID(String id) {
        log.debug("Request to get Statistics by ExerciseId : {}", id);
        return statisticsRepository.findByExerciseID(id).map(statisticsMapper::toDto);
    }

    @Override
    public void delete(Long id) {
        log.debug("Request to delete Statistics : {}", id);
        statisticsRepository.deleteById(id);
        statisticsSearchRepository.deleteById(id);
    }
    //    @Override
    //    @Transactional(readOnly = true)
    //    public Page<StatisticsDTO> search(String query, Pageable pageable) {
    //        log.debug("Request to search for a page of Statistics for query {}", query);
    //        return statisticsSearchRepository.search(queryStringQuery(query), pageable)
    //            .map(statisticsMapper::toDto);
    //    }

}
